;9.8.07 Anpassung der Baudrate

;7.11.05  bertragen auf ATtiny2313
;Brennen mit Brownout 2,7 V, Ext Crystal 3-8 MHz, Lockbits

;Relais-Interface
;Steuersequenz: 27,Adresse,Kommando,Parameter1,Parameter2
;Adresse 255 = General-ID
;27,Adr,64,Dat,0   direkte Portausgabe
;27,Adr,72,ID,0    umprogrammieren der ID-Adresse
;27,Adr,68,0,0     Programm Start
;27,Adr,70,0,0     Programm Stop
;27,Adr,66,EEadr,EEdat   EEprom programmieren 	
;27,Adr,74,0,0 	   Programm Start: einmal	

;Sync-Signal: Master setzt high whrend Bitmustre Nr. 1, Slave wartet
;Betriebsmodi: 0=direkt, 1=endlos, 2=einmal		

;EEPROM-Data befinden sich in WORKEE.ASM und werden mit programmiert


;Neu 21.7.04: Umstellung auf 2313, 60 Programmschritte
;Slave Autostart erst nach Sync = 1 
;RS232 unterbricht Wartezustand bei Sync = 0



.include "tn2313def.inc"
;.include "2313def.inc"
;.device AT90S1200

.def	Temp	=r16		;temporary register
.def	Delay	=r17
.def	Delay2	=r18
         
.def	A	=r19
.def	Count	=r20
.def	B	=r21

.def	Adr	=r23	;empfangene Adresse
.def	Kom	=r22	;Kommando
.def	Dat	=r25	;Daten
.def	ID	=r26	;eigende ID-Adresse	
.def	EEadr	=r27	;aktuelle EEPROM-Adresse
.def	Dat2	=r28	;zweites Parameterbyte

.def	Tlo	=r0	;Zeitregister low
.def	Thi	=r1	;Zeitregister high
.def	TiLo	=r2	;Intervall low
.def	TiHi	=r3	;Intervall high
.def	EErdAdr	=r4
.def	TCnt	=r5	;Zeitregister-Zhler
.def	TiCnt	=r6	;Intervallzeit-Zhler
.def	Modus	=r7	;Betriebsmodus =0/1/2
    
.equ	RXD	=2
.equ	TXD	=1
.equ	SYNC	=0


         ;***** Initialization
         
        
	rjmp	RESET
	rjmp	EXT_INT0
	rjmp	EXT_INT1
	rjmp	TIM_CAPT1
	rjmp	TIM_COMP1
	rjmp	TIM_OVF1
	rjmp	TIM_OVF0
	rjmp	UART_RXC
	rjmp	UART_DRE
	rjmp	UART_TXC
	rjmp	ANA_COMP

EXT_INT0: 	reti
EXT_INT1: 	reti
TIM_CAPT1: 	reti
TIM_COMP1: 	reti
TIM_OVF1: 	reti
;;;;TIM_OVF0: 	reti
UART_RXC: 	reti
UART_DRE: 	reti
UART_TXC: 	reti
ANA_COMP: 	reti


Reset:	cli
	ldi	A,RAMEND	;Stackpointer bei DFh
	out	SPL,A
	ldi	A,127
	out	EEDR,A		;Inhalt schtzen
 	ldi	Temp,255	;nur PB2...7 als Ausgnge
      	out	DDRB,Temp	;PORTB = all outputs
	out	DDRD,Temp
      	sbi	DDRD,TXD
	cbi	DDRD,RXD
	cbi	DDRD,SYNC	;Sync Eingang
	sbi	PORTD,SYNC	;mit Pullup
	ldi	A,0
	out	PORTB,A
	ldi	Delay,0
	ldi	Delay2,0
DLY:	dec	Delay
	breq	DLY
	dec	Delay2
	breq	DLY		;Wartezeit am Start wg. EEPROM
	ldi	A,0		;Timer Stop
	out	TCCR0,A
	ldi	Temp, 0
	out	TCNT0,Temp	;Timer = 0
	ldi	A,2
	out	TIMSK,A		;Timer Interrupt freigeben
	ldi	A,5		;Prescaler /1024; Start
	out	TCCR0,A
	ldi	EEadr,0		;EEPROM,0: ID-Adresse
	rcall	RdEE
	mov	ID,A
	cpi	A,1
	brne	SLSyn
MasSyn: sbi	DDRD,SYNC	;Master SYNC High
	sbi	PORTD,SYNC
SLSyn:	inc	EEadr		;Slave: SYNC Eingang
	rcall	RdEE		;EE, Adr1, Modus:0,1,2
	mov	Modus,A
	cpi	A,0
	breq	Main
	rcall	Auto

Main:	rcall 	RdCOM
	cpi	A,27
	brne	Main
	rcall	RdCOM
	mov	Adr,A
	rcall	RdCOM
	mov	Kom,A
	rcall	RdCOM
	mov	Dat,A
	rcall	RdCOM
	mov	Dat2,A
Adres:	cpi	Adr,255		;Generaladresse 255?
	breq	K64
	cp	Adr,ID		;ID?
	brne	Main
K64:	cpi	Kom,64		;64: direkte Portausgabe
	brne	K66
	out 	PORTB,Dat
	rcall 	WrCOM
K66:	cpi	Kom,66
	brne	K68
	mov	EEadr,Dat
	mov	A,Dat2
	rcall	WrEE
K68:	cpi	Kom,68		;Programmstart
	brne	K70
	clr	Thi 		;Vorbereiten fr nchsten berlauf
	clr	Tlo
	clr	TCnt
	ldi	EEadr,2
	rcall	RdEE
	mov	TiHi,A		;Timer-Intervall, high
	ldi	EEadr,3
	rcall	RdEE
	mov	TiLo,A		;Timer-Intervall, low
	ldi	EEadr,4
	rcall	RdEE
	out	PORTB,A		;erstes Datum
	cpi	ID,1		;Adresse 1: Master
	brne	Sl68
M68:	sbi	PORTD,SYNC 
SL68:	ldi	EEadr,5
	rcall	RdEE
	mov	TiCnt,A		;erste Wartezeit
	mov	EErdAdr,EEAdr
	inc	EErdAdr
	ldi	EEadr,1		;Startmakre 1 speichern
	ldi	A,1
	rcall	WrEE
	mov	Modus,A
	ldi	A,5		;Timer Start
	out	TCCR0,A
	sei			;Timer-Interrupt freigeben
K70:	cpi	Kom,70
	brne	K72
	cli
	ldi	EEadr,1		;Startmakre 0 speichern
	ldi	A,0
	rcall	WrEE
K72:	cpi	Kom,72		;72: ID programmieren
	brne	K74
	mov	ID,Dat
	ldi	EEadr,0
	mov	A,Dat
	rcall	WrEE		;ID speichern, 2,5 ms
	cpi	A,1		;Master:
	brne	SID
MID:	sbi	DDRD,SYNC	;Master SYNC High
	sbi	PORTD,SYNC
	rjmp	K74
SID:	cbi	DDRD,SYNC	;Slave SYNC Eingang
	sbi	PORTD,SYNC	;mit Pullup
K74:	cpi	Kom,74		;Programmstart
	brne	K76
	clr	Thi 		;Vorbereiten fr nchsten berlauf
	clr	Tlo
	clr	TCnt
	ldi	EEadr,2
	rcall	RdEE
	mov	TiHi,A		;Timer-Intervall, high
	ldi	EEadr,3
	rcall	RdEE
	mov	TiLo,A		;Timer-Intervall, low
	ldi	EEadr,4
	rcall	RdEE
	out	PORTB,A		;erstes Datum
	cpi	ID,1		;Adresse 1: Master
	brne	Sl74
M74:	sbi	PORTD,SYNC 
SL74:	ldi	EEadr,5
	rcall	RdEE
	mov	TiCnt,A		;erste Wartezeit
	mov	EErdAdr,EEAdr
	inc	EErdAdr
	ldi	EEadr,1		;Startmakre 1 speichern
	ldi	A,2
	rcall	WrEE
	mov	Modus,A
	ldi	A,5		;Timer Start
	out	TCCR0,A
	sei			;Timer-Interrupt freigeben
K76:	rjmp 	Main

RdCOM:	sbis	PIND,RXD   ;9,6
	rjmp	RdCOM
	ldi	Delay,200
D1:	dec 	Delay
	brne	D1
	ldi	A,0
	ldi	Count,8
L1:	lsr	A           ;1
	sbic	PIND,RXD  ;2
	ori	A,128          
	ldi	Delay, 143   ;136   ;1           143 !!!
D2:	dec 	Delay   ;1 * 136
	brne	D2      ;2 * 136
	dec	Count       ;1
	brne	L1      ;2  ;3*136 + 6 = 414   3*143+6 =435, 4194 kHz/426 = 9,684 kBaud
	ldi	Delay, 86   ;50	;Stopbit verkrzt, min 286-200 = 86 !!
D3:	dec	Delay
	brne	D3
	com	A
	ret

WrCOM:	sbi	PORTD,TXD
	ldi	Delay,136
D4:	dec	Delay
	brne	D4
	ldi	Count,8
L2:	sbrc	A,0
	rjmp	OFF
	rjmp	ON
ON:	sbi	PORTD,TXD
	rjmp	BitD
OFF:	cbi	PORTD,TXD
	rjmp	BitD
BitD:	ldi	Delay,136
D5:	dec	Delay
	brne	D5
	lsr	A
	dec	Count
	brne	L2
	cbi	PORTD,TXD
	ret


RdEE:	sbic	EECR,EEWE	;Programmtokens lesen
	rjmp	RdEE
	out	EEAR,EEadr
	sbi	EECR,EERE
	in	A,EEDR
	ret

WrEE: 	sbic	EECR,EEWE
	rjmp	WrEE
	out	EEAR,EEadr
	out	EEDR,A
	sbi	EECR,EEMWE	;Neu beim 2313: Master Write Enable
	sbi	EECR,EEWE	;dauert ca. 2,5 ms bei 5V bis fertig
	ret
	
Auto:		;Neu: Slave wartet auf Sync=1
	cpi	ID,1		;Adresse 1: Master
	brne	SlavA
	rjmp	AutoW		;Master, gleich weiter
SlavA:	sbic	PIND,RXD
	ret			;beenden bei RXD-Signal
	sbis	PIND,SYNC	;Sync=1 abwarten
 	rjmp	SlavA

Autow:	clr	Thi 		;Vorbereiten fr nchsten berlauf
	clr	Tlo
	clr	TCnt
	ldi	EEadr,2
	rcall	RdEE
	mov	TiHi,A		;Timer-Intervall, high
	ldi	EEadr,3
	rcall	RdEE
	mov	TiLo,A		;Timer-Intervall, low
	ldi	EEadr,4
	rcall	RdEE
	out	PORTB,A		;erstes Datum
	ldi	EEadr,5
	rcall	RdEE
	mov	TiCnt,A		;erste Wartezeit
	mov	EErdAdr,EEAdr
	inc	EErdAdr
	sei			;Timer-Interrupt freigeben
	ret
	
	

TIM_OVF0: cpi	ID,1
	brne	SL
	cbi	PORTD,SYNC
SL:	inc	Tlo	;Zeit lo
	brne	Vgl	;bertrag wenn Tlo=0
	inc 	Thi	;Zeit Hi
Vgl:	cp	Thi,TiHi ;
	breq	VglLo
	reti
VglLo:	cp	Tlo,TiLo
	breq	VglCnt
	reti
VglCnt:	clr	Thi
	clr	Tlo
	inc	Tcnt
	cp	TCnt,TiCnt
	breq	Output
	reti

OutPut:	sbic	EECR,EEWE
	rjmp	OutPut
	mov	Temp,EErdAdr
	cpi	Temp,64		;Speicherende erreicht?
	breq	Out1
	clr	TCnt
	out	EEAR,EErdAdr
	sbi	EECR,EERE
	sbi	EECR,EERE
	in	Temp,EEDR	;Datum holen
	inc	EErdAdr
RdTCnt:	sbic	EECR,EEWE
	rjmp	RdTCnt
	out	EEAR,EErdAdr
	sbi	EECR,EERE
	sbi	EECR,EERE
	in	TiCnt,EEDR	;Wartezeit holen
	inc	EErdAdr
	cp	TiCnt,Tcnt	;vergleiche letztes Byte mit 0
	breq	Out1		;Ende der Musterserie?
	out	PORTB,Temp	;Ausgabe wenn nicht Ende
	reti
Loopend: ldi	A,0		;Timer Stop
	out	TCCR0,A
	reti
Out1:	mov	Temp,Modus
	cpi	Temp,1
	brne	Loopend		;wenn Modus = 2, single output
	cpi	ID,1		;Adresse 1: Master
	brne	Slave
Master:	sbi	PORTD,SYNC 
Slave:	sbis	PIND,SYNC	;Sync=1 abwarten
	rjmp	Slave
	ldi	Temp,4		;erste EE-Adresse
	mov	EErdAdr,Temp
OutP1:	sbic	EECR,EEWE
	rjmp	OutP1
	clr	TCnt
	out	EEAR,EErdAdr
	sbi	EECR,EERE
	sbi	EECR,EERE
	in	Temp,EEDR	;Datum holen
	inc	EErdAdr
RdTC1:	sbic	EECR,EEWE
	rjmp	RdTC1
	out	EEAR,EErdAdr
	sbi	EECR,EERE
	sbi	EECR,EERE
	in	TiCnt,EEDR	;Wartezeit holen
	inc	EErdAdr
	out	PORTB,Temp	;Ausgabe wenn nicht Ende
	reti
	

 
